@@ -13,13 +13,12 @@
 #include "iostream.hpp"
 
 namespace acommon {
-  unsigned int linenumber = 0 ;
 
   bool getdata_pair(IStream & in, DataPair & d, String & buf)
   {
     char * p;
 
-    // get first non blank line and count all read ones
+    // get first non blank line
     do {
       buf.clear();
       buf.append('\0'); // to avoid some special cases
@@ -41,7 +40,7 @@ namespace acommon {
     if (*p == '#' || *p == '\0') {*p = '\0'; return true;}
     *p = '\0';
 
-    // skip any whitespace
+    // skip any whitspace
     ++p;
     while (*p == ' ' || *p == '\t') ++p;
     if (*p == '\0' || *p == '#') {return true;}
@@ -64,7 +63,7 @@ namespace acommon {
   char * unescape(char * dest, const char * src)
   {
     while (*src) {
-      if (*src == '\\' && src[1]) {
+      if (*src == '\\') {
 	++src;
 	switch (*src) {
 	case 'n': *dest = '\n'; break;
@@ -86,14 +85,7 @@ namespace acommon {
 
   bool escape(char * dest, const char * src, size_t limit, const char * others)
   {
-    const char * begin = src;
     const char * end = dest + limit;
-    if (asc_isspace(*src)) {
-      if (dest == end) return false;
-      *dest++ = '\\';
-      if (dest == end) return false;
-      *dest++ = *src++;
-    }
     while (*src) {
       if (dest == end) return false;
       switch (*src) {
@@ -111,12 +103,6 @@ namespace acommon {
       ++src;
       ++dest;
     }
-    if (src > begin + 1 && asc_isspace(src[-1])) {
-      --dest;
-      *dest++ = '\\';
-      if (dest == end) return false;
-      *dest++ = src[-1];
-    }
     *dest = '\0';
     return true;
   }
@@ -126,11 +112,6 @@ namespace acommon {
     for (; *str; str++) *str = asc_tolower(*str);
   }
 
-  void to_lower(String & res, const char * str)
-  {
-    for (; *str; str++) res += asc_tolower(*str);
-  }
-
   bool split(DataPair & d)
   {
     char * p   = d.value;
@@ -169,25 +150,4 @@ namespace acommon {
     return true;
   }
 
-  char * get_nb_line(IStream & in, String & buf)
-  {
-    char * p;
-    // get first non blank line
-    do {
-      if (!in.getline(buf)) return 0;
-      p = buf.mstr();
-      while (*p == ' ' || *p == '\t') ++p;
-    } while (*p == '#' || *p == '\0');
-    return p;
-  }
-
-  void remove_comments(String & buf)
-  {
-    char * p = buf.mstr();
-    char * b = p;
-    while (*p && *p != '#') ++p;
-    if (*p == '#') {--p; while (p >= b && asc_isspace(*p)) --p; ++p;}
-    buf.resize(p - b);
-  }
-
 }
