@@ -3,6 +3,8 @@
 
 namespace acommon {
 
+using namespace std;
+
 void ObjStack::setup_chunk()
 {
   bottom = first_free->data;
@@ -13,7 +15,7 @@ void ObjStack::setup_chunk()
 
 
 ObjStack::ObjStack(size_t chunk_s, size_t align)
-  : chunk_size(chunk_s), min_align(align), temp_end(0)
+  : chunk_size(chunk_s), min_align(align)
 {
   first_free = first = (Node *)malloc(chunk_size);
   first->next = 0;
@@ -31,14 +33,6 @@ ObjStack::~ObjStack()
   trim();
 }
 
-size_t ObjStack::calc_size()
-{
-  size_t size = 0;
-  for (Node * p = first; p; p = p->next)
-    size += chunk_size;
-  return size;
-}
-
 void ObjStack::new_chunk()
 {
   if (reserve) {
@@ -56,12 +50,8 @@ void ObjStack::new_chunk()
 
 void ObjStack::reset()
 {
-  assert(first_free->next == 0);
-  if (first->next) {
-    first_free->next = reserve;
-    reserve = first->next;
-    first->next = 0;
-  } 
+  reserve = first->next;
+  first->next = 0;
   first_free = first;
   setup_chunk();
 }
@@ -75,23 +65,4 @@ void ObjStack::trim()
   }
 }
 
-ObjStack::Memory * ObjStack::freeze() {
-  trim();
-  Node * tmp = first;
-  first = 0;
-  first_free = 0;
-  top = 0;
-  bottom = 0;
-  temp_end = 0;
-  return tmp;
-}
-  
-void ObjStack::dealloc(Memory * ptr) {
-  while (ptr) {
-    Node * tmp = ptr->next;
-    free(ptr);
-    ptr = tmp;
-  }
-}
-
 }
