@@ -5,7 +5,7 @@
 // it at http://www.gnu.org/.
 
 // Aspell implementation header file.
-// Applications that just use the Aspell library should not include 
+// Applications that just use the aspell library should not include 
 // these files as they are subject to change.
 // Aspell Modules MUST include some of the implementation files and
 // spell checkers MAY include some of these files.
@@ -56,8 +56,8 @@ namespace acommon {
     Speller(const Speller &);
     Speller & operator= (const Speller &);
   public:
-    String temp_str_0;
-    String temp_str_1;
+    CharVector temp_str_0;
+    CharVector temp_str_1;
     ClonePtr<Convert> to_internal_;
     ClonePtr<Convert> from_internal_;
   protected:
