@@ -14,15 +14,13 @@ print OUT <<'---';
 #. not include any trailing punctuation marks.
 ---
 
-@files = (<modules/filter/*-filter.info>, <modules/filter/modes/*.amf>);
-
-foreach $f (@files)
+foreach $f (<modules/filter/*-filter.opt>)
 {
   open IN, $f;
   while (<IN>) 
   {
-    next unless /^\s*(description|desc)\s+(.+?)\s*$/i;
-    $_ = prep_str($2);
+    next unless /^\s*description\s+(.+?)\s*$/i;
+    $_ = prep_str($1);
     print OUT "\#: $f:$.\n";
     print OUT "msgid \"$_\"\n";
     print OUT "msgstr \"\"\n";
