@@ -4,17 +4,14 @@
 // license along with this library if you did not you can find
 // it at http://www.gnu.org/.
 
-#include "settings.h"
-
-#include "indiv_filter.hpp"
 #include "convert.hpp"
 #include "config.hpp"
 #include "indiv_filter.hpp"
 #include "mutable_container.hpp"
+#include "copy_ptr-t.hpp"
+#include "loadable-filter-API.hpp"
 
-namespace {
-
-  using namespace acommon;
+namespace acommon {
 
   class EmailFilter : public IndividualFilter 
   {
@@ -27,22 +24,20 @@ namespace {
     public:
       typedef FilterChar::Chr Value;
       Vector<Value> data;
-      ConvEC conv;
+      Conv conv;
       bool have(Value c) {
         Value * i = data.pbegin();
         Value * end = data.pend();
         for (; i != end && *i != c; ++i);
         return i != end;
       }
-      PosibErr<bool> add(ParmStr s) {
-        RET_ON_ERR_SET(conv(s), const char *, val);
-        Value c = *(Value *)val;
+      PosibErr<bool> add(ParmString s) {
+        Value c = *(Value *)conv(s);
         if (!have(c)) data.push_back(c);
         return true;
       }
-      PosibErr<bool> remove(ParmStr s) {
-        RET_ON_ERR_SET(conv(s), const char *, val);
-        Value c = *(Value *)val;
+      PosibErr<bool> remove(ParmString s) {
+        Value c = *(Value *)conv(s);
         Vector<Value>::iterator i = data.begin();
         Vector<Value>::iterator end = data.end();
         for (; i != end && *i != c; ++i);
@@ -66,9 +61,9 @@ namespace {
   {
     name_ = "email-filter";
     order_num_ = 0.85;
-    is_quote_char.conv.setup(*opts, "utf-8", ConvKey("ucs-4", true), NormNone);
-    RET_ON_ERR(opts->retrieve_list("f-email-quote", &is_quote_char));
-    margin = opts->retrieve_int("f-email-margin");
+    is_quote_char.conv.setup(*opts, "utf-8", "utf-32");
+    opts->retrieve_list("filter-email-quote", &is_quote_char);
+    margin = opts->retrieve_int("filter-email-margin");
     reset();
     return true;
   }
@@ -107,11 +102,7 @@ namespace {
       for (FilterChar * i = line_begin; i != cur; ++i)
 	*i = ' ';
   }
+  
+ACTIVATE_FILTER(acommon,EmailFilter,email);
 }
 
-C_EXPORT 
-IndividualFilter * new_aspell_email_filter() {
-  return new EmailFilter;                                
-}
-
-
