@@ -1,5 +1,5 @@
 // This file is part of The New Aspell
-// Copyright (C) 2000-2001,2011 by Kevin Atkinson under the GNU LGPL
+// Copyright (C) 2000-2001 by Kevin Atkinson under the GNU LGPL
 // license version 2.0 or 2.1.  You should have received a copy of the
 // LGPL license along with this library if you did not you can find it
 // at http://www.gnu.org/.
@@ -16,7 +16,7 @@
 #include "cache-t.hpp"
 #include "vararray.hpp"
 
-#include "gettext.h"
+#include "iostream.hpp"
 
 namespace aspeller {
 
@@ -69,8 +69,7 @@ namespace aspeller {
   Dictionary::Dictionary(BasicType t, const char * n)
     : Cacheable(&dict_cache), lang_(), id_(), 
       basic_type(t), class_name(n),
-      affix_compressed(false), 
-      invisible_soundslike(false), soundslike_root_only(false),
+      affix_compressed(false), have_soundslike(false), 
       fast_scan(false), fast_lookup(false)
   {
     id_.reset(new Id(this));
@@ -90,13 +89,12 @@ namespace aspeller {
     return no_err;
   }
  
-  PosibErr<void> Dictionary::set_check_lang (ParmString l, Config & config)
+  PosibErr<void> Dictionary::set_check_lang (ParmString l, const Config * config)
   {
     if (lang_ == 0) {
-      PosibErr<Language *> res = new_language(config, l);
+      PosibErr<Language *> res = new_language(*config, l);
       if (res.has_err()) return res;
       lang_.reset(res.data);
-      RET_ON_ERR(lang_->set_lang_defaults(config));
       set_lang_hook(config);
     } else {
       if (l != lang_->name())
@@ -154,32 +152,31 @@ namespace aspeller {
   // BasicDict
   //
 
-  class DictStringEnumeration : public StringEnumeration 
-  {
-    ClonePtr<Dict::Enum> real_;
-  public:
-    DictStringEnumeration(Dict::Enum * r) : real_(r) {}
-    
-    bool at_end() const {
-      return real_->at_end();
-    }
-    const char * next() {
-      // FIXME: It's not this simple when affixes are involved
-      WordEntry * w =  real_->next(); 
-      if (!w) return 0;
-      return w->word;
-    }
-    StringEnumeration * clone() const {
-      return new DictStringEnumeration(*this);
-    }
-    void assign(const StringEnumeration * other) {
-      *this = *static_cast<const DictStringEnumeration *>(other);
-    }
-  };
+//   class BasicDictEnumeration : public StringEnumeration 
+//   {
+//     BasicDict::Emul real_;
+//   public:
+//     BasicDictEnumeration(BasicDict::VirEmul * r) : real_(r) {}
+
+//     bool at_end() const {
+//       return real_.at_end();
+//     }
+//     const char * next() {
+//       return real_.next().word; // FIXME: It's not this simple
+//     }
+//     StringEnumeration * clone() const {
+//       return new BasicDictEnumeration(*this);
+//     }
+//     void assign(const StringEnumeration * other) {
+//       *this = *static_cast<const BasicDictEnumeration *>(other);
+//     }
+//   };
+
+
 
   PosibErr<void> Dictionary::add_repl(ParmString mis, ParmString cor) 
   {
-    if (!invisible_soundslike) {
+    if (have_soundslike) {
       VARARRAY(char, sl, mis.size() + 1);
       lang()->LangImpl::to_soundslike(sl, mis.str(), mis.size());
       return add_repl(mis, cor, sl);
@@ -190,7 +187,7 @@ namespace aspeller {
 
   PosibErr<void> Dictionary::add(ParmString w) 
   {
-    if (!invisible_soundslike) {
+    if (have_soundslike) {
       VARARRAY(char, sl, w.size() + 1);
       lang()->LangImpl::to_soundslike(sl, w.str(), w.size());
       return add(w, sl);
@@ -203,8 +200,9 @@ namespace aspeller {
   // Default implementation;
   //
 
-  PosibErr<void> Dictionary::load(ParmString, Config &, 
-                                  DictList *, SpellerImpl *) 
+  PosibErr<void> Dictionary::load(ParmString, const Config &, 
+                                  LocalDictList *, 
+                                  SpellerImpl *, const LocalDictInfo *) 
   {
     return make_err(unimplemented_method, "load", class_name);
   }
@@ -236,9 +234,7 @@ namespace aspeller {
 
   StringEnumeration * Dictionary::elements() const
   {
-    Enum * e = detailed_elements();
-    if (!e) return 0;
-    return new DictStringEnumeration(e);
+    return 0;
   }
   
   Dict::Enum * Dictionary::detailed_elements() const
@@ -257,8 +253,8 @@ namespace aspeller {
     return false;
   }
   
-  bool Dictionary::lookup (ParmString word, const SensitiveCompare *,
-                           WordEntry &) const
+  bool Dictionary::lookup (ParmString word, WordEntry &,
+                           const SensitiveCompare &) const
   {
     return false;
   }
@@ -313,22 +309,25 @@ namespace aspeller {
     return make_err(unimplemented_method, "remove_repl", class_name);
   }
   
-  DictsEnumeration * Dictionary::dictionaries() const 
+  Enumeration<const LocalDict *> * Dictionary::dictionaries() const 
   {
     return 0;
   }
 
 #define write_conv(s) do { \
     if (!c) {o << s;} \
-    else {ParmString ss(s); buf.clear(); c->convert(ss.str(), ss.size(), buf); o.write(buf.data(), buf.size());} \
+    else {ParmString ss(s); buf.clear(); c->convert(ss.str(), ss.size(), buf); o.write(buf.data(), buf.size()-1);} \
   } while (false)
 
   OStream & WordEntry::write (OStream & o,
                               const Language & l,
+                              const ConvertWord & cw,
                               Convert * c) const
   {
+    String w;
     CharVector buf;
-    write_conv(word);
+    cw.convert(word, w);
+    write_conv(w);
     if (aff && *aff) {
       o << '/';
       write_conv(aff);
@@ -336,14 +335,15 @@ namespace aspeller {
     return o;
   }
 
-  PosibErr<Dict *> add_data_set(ParmString fn,
-                                Config & config,
-                                DictList * new_dicts,
-                                SpellerImpl * speller,
-                                ParmString dir,
-                                DataType allowed)
+  PosibErr<void> add_data_set(ParmString fn,
+                              const Config & config,
+                              LocalDict & res,
+                              LocalDictList * new_dicts,
+                              SpellerImpl * speller,
+                              const LocalDictInfo * local_info,
+                              ParmString dir,
+                              DataType allowed)
   {
-    Dict * res = 0;
     static const char * suffix_list[] = {"", ".multi", ".alias", 
 					 ".spcl", ".special",
 					 ".pws", ".prepl"};
@@ -387,7 +387,7 @@ namespace aspeller {
 
     } else {
       
-      char head[32] = {0};
+      char head[32];
       in.read(head, 32);
       if      (strncmp(head, "aspell default speller rowl", 27) ==0)
 	actual_type = DT_ReadOnly;
@@ -397,32 +397,30 @@ namespace aspeller {
 	actual_type = DT_Writable;
       else
 	return make_err(bad_file_format, true_file_name);
+
     }
     
     if (actual_type & ~allowed)
       return make_err(bad_file_format, true_file_name
 		      , _("is not one of the allowed types"));
 
-    Dict::FileName id_fn(true_file_name);
-    Dict::Id id(0,id_fn);
+    Dict::Id id(0,Dict::FileName(true_file_name));
 
     if (speller != 0) {
-      const SpellerDict * d = speller->locate(id);
+      const LocalDict * d = speller->locate(id);
       if (d != 0) {
-        return d->dict;
+        res = *d;
+        return no_err;
       }
     }
 
-    res = 0;
-
-    Lock dict_cache_lock(NULL);
+    res.dict = 0;
 
     if (actual_type == DT_ReadOnly) { // try to get it from the cache
-      dict_cache_lock.set(&dict_cache.lock); 
-      res = dict_cache.find(id);
+      res.dict = dict_cache.find(id);
     }
 
-    if (!res) {
+    if (!res.dict) {
 
       StackPtr<Dict> w;
       switch (actual_type) {
@@ -442,27 +440,52 @@ namespace aspeller {
         abort();
       }
 
-      RET_ON_ERR(w->load(true_file_name, config, new_dicts, speller));
+      RET_ON_ERR(w->load(true_file_name, config, new_dicts, speller, local_info));
 
       if (actual_type == DT_ReadOnly)
         dict_cache.add(w);
       
-      res = w.release();
+      res.dict = w.release();
 
-    } else { // actual_type == DT_ReadOnly implied, and hence the lock
-             // is already acquired
-
-      res->copy_no_lock();
+    } else {
+      
+      res.dict->copy();
       
     }
 
-    dict_cache_lock.release();
-
+    if (local_info) {
+      res.set(*local_info);
+      res.set_language(res.dict->lang());
+    } else
+      res.set(res.dict->lang(), config);
     if (new_dicts)
       new_dicts->add(res);
     
-    return res;
+    return no_err;
   }
 
+  //
+  // LocalDictInfo
+  //
+  
+  void LocalDictInfo::set_language(const Language * l)
+  {
+    compare.lang = l;
+    convert.lang = l;
+  }
+
+  void LocalDictInfo::set(const Language * l, const Config & c, bool strip)
+  {
+    if (c.have("strip-accents"))
+      strip = c.retrieve_bool("strip-accents");
+
+    compare.lang = l;
+    compare.case_insensitive = c.retrieve_bool("ignore-case");
+    compare.ignore_accents   = c.retrieve_bool("ignore-accents");
+    compare.strip_accents    = strip;
+    convert.lang = l;
+    convert.strip_accents = strip;
+  }
+  
 }
 
