@@ -1,4 +1,4 @@
-// Copyright 2004 by Kevin Atkinson under the terms of the LGPL
+// Copyright 2000 by Kevin Atkinson under the terms of the LGPL
 
 #ifndef ASPELLER_LANGUAGE__HPP
 #define ASPELLER_LANGUAGE__HPP
@@ -12,29 +12,11 @@
 #include "stack_ptr.hpp"
 #include "string.hpp"
 #include "objstack.hpp"
-#include "string_enumeration.hpp"
-
-#include "iostream.hpp"
 
 using namespace acommon;
 
 namespace acommon {
   struct CheckInfo;
-
-  struct ConfigConvKey : public ConvKey {
-    Config::Value config_val;
-    template <typename T>
-    ConfigConvKey(const T & v) : config_val(v) {
-      val = config_val.val;
-      allow_ucs = config_val.secure;
-    }
-    void fix_encoding_str() {
-      String buf;
-      ::fix_encoding_str(val, buf);
-      config_val.val.swap(buf);
-      val = config_val.val;
-    }
-  };
 }
 
 namespace aspeller {
@@ -58,43 +40,36 @@ namespace aspeller {
     }
   };
 
+  // WordInfo
+
+  typedef unsigned int WordInfo; // 4 bits
+
+  enum CasePattern {Other, FirstUpper, AllLower, AllUpper};
+  //   Other      00
+  //   FirstUpper 01
+  //   AllLower   10
+  //   AllUpper   11
+  // First bit : is upper
+  // Second bit: uniform case
+
+  static const WordInfo CASE_PATTERN = 3;
+  static const WordInfo ALL_PLAIN    = (1 << 2);
+  static const WordInfo ALL_LETTER   = (1 << 3);
+
   // CharInfo
 
-  typedef unsigned int CharInfo; // 6 bits
+  typedef unsigned int CharInfo; // 5 bits
 
   static const CharInfo LOWER  = (1 << 0);
   static const CharInfo UPPER  = (1 << 1);
   static const CharInfo TITLE  = (1 << 2);
   static const CharInfo PLAIN  = (1 << 3);
   static const CharInfo LETTER = (1 << 4);
-  static const CharInfo CLEAN  = (1 << 5);
 
-  static const CharInfo CHAR_INFO_ALL = 0x3F;
+  static const CharInfo CHAR_INFO_ALL = 0x1F;
 
   //
 
-  //
-
-  struct CompoundWord {
-    const char * word;
-    const char * sep;
-    const char * rest;
-    const char * end;
-    bool empty() const {return word == end;}
-    bool single() const {return rest == end;}
-    unsigned word_len() const {return sep - word;}
-    unsigned rest_offset() const {return rest - word;}
-    unsigned rest_len() const {return end - rest;}
-    CompoundWord()
-      : word(), sep(), rest(), end() {}
-    CompoundWord(const char * a, const char * b)
-      : word(a), sep(b), rest(b), end(b) {}
-    CompoundWord(const char * a, const char * b, const char * c)
-      : word(a), sep(b), rest(b), end(c) {}
-    CompoundWord(const char * a, const char * b, const char * c, const char * d)
-      : word(a), sep(b), rest(c), end(d) {}
-  };
-
   enum StoreAs {Stripped, Lower};
 
   class Language : public Cacheable {
@@ -103,8 +78,8 @@ namespace aspeller {
     typedef String       CacheKey;
 
     enum CharType {Unknown, WhiteSpace, Hyphen, Digit, 
-                   NonLetter, Modifier, Letter};
-    
+                   NonLetter, Modifier, Vowel, Letter};
+
     struct SpecialChar {
       bool begin;
       bool middle;
@@ -119,7 +94,6 @@ namespace aspeller {
     String   dir_;
     String   name_;
     String   charset_;
-    String   charmap_;
     String   data_encoding_;
 
     ConvObj  mesg_conv_;
@@ -134,20 +108,20 @@ namespace aspeller {
     char          to_upper_[256];
     char          to_title_[256];
     char          to_stripped_[256];
+    unsigned char to_normalized_[256];
     char          to_plain_[256];
+    char          to_sl_[256];
     int           to_uni_[256];
     CharType      char_type_[256];
-    char          to_clean_[256];
-    char          de_accent_[256];
+    char *        to_clean_;
 
     StoreAs       store_as_;
 
+    int max_normalized_;
+
     String      soundslike_chars_;
     String      clean_chars_;
 
-    bool have_soundslike_;
-    bool have_repl_;
-
     StackPtr<Soundslike> soundslike_;
     StackPtr<AffixMgr>   affix_;
     StackPtr<Config>     lang_config_;
@@ -158,32 +132,21 @@ namespace aspeller {
     Language(const Language &);
     void operator=(const Language &);
 
-  public: // but don't use
-
-    char          sl_first_[256];
-    char          sl_rest_[256];
-
   public:
 
     Language() {}
     PosibErr<void> setup(const String & lang, const Config * config);
-    PosibErr<void> set_lang_defaults(Config & config) const;
+    void set_lang_defaults(Config & config);
 
     const char * data_dir() const {return dir_.c_str();}
     const char * name() const {return name_.c_str();}
-    const char * charmap() const {return charmap_.c_str();}
+    const char * charset() const {return charset_.c_str();}
     const char * data_encoding() const {return data_encoding_.c_str();}
 
     const Convert * mesg_conv() const {return mesg_conv_.ptr;}
     const Convert * to_utf8() const {return to_utf8_.ptr;}
     const Convert * from_utf8() const {return from_utf8_.ptr;}
 
-    int to_uni(char c) const {return to_uni_[to_uchar(c)];}
-
-    //
-    // case conversion
-    //
-
     char to_upper(char c) const {return to_upper_[to_uchar(c)];}
     bool is_upper(char c) const {return to_upper(c) == c;}
 
@@ -193,174 +156,117 @@ namespace aspeller {
     char to_title(char c) const {return to_title_[to_uchar(c)];}
     bool is_title(char c) const {return to_title(c) == c;}
 
-    char * to_lower(char * res, const char * str) const {
-      while (*str) *res++ = to_lower(*str++); *res = '\0'; return res;}
-    char * to_upper(char * res, const char * str) const {
-      while (*str) *res++ = to_upper(*str++); *res = '\0'; return res;}
-
-    void to_lower(String & res, const char * str) const {
-      res.clear(); while (*str) res += to_lower(*str++);}
-    void to_upper(String & res, const char * str) const {
-      res.clear(); while (*str) res += to_upper(*str++);}
-
-    bool is_lower(const char * str) const {
-      while (*str) {if (!is_lower(*str++)) return false;} return true;}
-    bool is_upper(const char * str) const {
-      while (*str) {if (!is_upper(*str++)) return false;} return true;}
-
-    //
-    //
-    //
+    char to_stripped(char c) const {return to_stripped_[to_uchar(c)];}
+    bool is_stripped(char c) const {return to_stripped(c) == c;}
 
+    char to_normalized(char c) const {return to_normalized_[to_uchar(c)];}
+    unsigned char max_normalized() const {return max_normalized_;}
+    
     char to_plain(char c) const {return to_plain_[to_uchar(c)];}
 
-    char de_accent(char c) const {return de_accent_[to_uchar(c)];}
-    
+    char to_sl(char c) const {return to_sl_[to_uchar(c)];}
+
+    char to_clean(char c) const {return to_clean_[to_uchar(c)];}
+    bool is_clean(char c) const {return to_clean(c) == c;}
+  
+    int to_uni(char c) const {return to_uni_[to_uchar(c)];}
+  
     SpecialChar special(char c) const {return special_[to_uchar(c)];}
   
     CharType char_type(char c) const {return char_type_[to_uchar(c)];}
     bool is_alpha(char c) const {return char_type(c) >  NonLetter;}
 
     CharInfo char_info(char c) const {return char_info_[to_uchar(c)];}
-
-    //
-    // stripped
-    //
-
-    char to_stripped(char c) const {return to_stripped_[to_uchar(c)];}
-
-    // return a pointer to the END of the string
-    char * to_stripped(char * res, const char * str) const {
-      for (; *str; ++str) {
-        char c = to_stripped(*str);
-        if (c) *res++ = c;
-      }
-      *res = '\0';
-      return res;
-    }
-    void to_stripped(String & res, const char * str) const {
-      res.clear();
-      for (; *str; ++str) {
-        char c = to_stripped(*str);
-        if (c) res += c;
-      }
-    }
-
-    bool is_stripped(char c) const {return to_stripped(c) == c;}
-
-    bool is_stripped(const char * str) const {
-      while (*str) {if (!is_stripped(*str++)) return false;} return true;}
-
-    //
-    // Clean
-    //
-    // The "clean" form is how words are indixed in the dictionary.
-    // It will at very least convert the word to lower case.  It may
-    // also strip accents and non-letters.
-    //
-
-    char to_clean(char c) const {return to_clean_[to_uchar(c)];}
-
-    char * to_clean(char * res, const char * str) const {
-      for (; *str; ++str) {
-        char c = to_clean(*str);
-        if (c) *res++ = c;
-      }
-      *res = '\0';
-      return res;
-    }
-    void to_clean(String & res, const char * str) const {
-      res.clear();
-      for (; *str; ++str) {
-        char c = to_clean(*str);
-        if (c) res += c;
-      }
-    }
-
-    bool is_clean(char c) const {return to_clean(c) == c;}
-
-    bool is_clean(const char * str) const {
-      while (*str) {if (!is_clean(*str++)) return false;} return true;}
-
-    bool is_clean_wi(WordInfo wi) const {
-      return false;
-      //return wi & CASE_PATTEN == AllLower && 
-    }
-
-
-    const char * clean_chars() const {return clean_chars_.c_str();}
-
-    //
-    // Soundslike
-    // 
   
-    bool have_soundslike() const {return have_soundslike_;}
-    
-    const char * soundslike_name() const {return soundslike_->name();}
-    const char * soundslike_version() const {return soundslike_->version();}
-
-    void to_soundslike(String & res, ParmStr word) const {
+    const char * to_soundslike(String & res, ParmString word) const {
       res.resize(word.size());
       char * e = soundslike_->to_soundslike(res.data(), word.str(), word.size());
       res.resize(e - res.data());
+      return res.str();
+    }
+
+    const char * soundslike_name() const {
+      return soundslike_->name();
+    }
+    bool have_soundslike() const {return strcmp(soundslike_->name(),"none") == 0;}
+
+    const char * soundslike_version() const {
+      return soundslike_->version();
     }
     
-    // returns a pointer to the END of the string
-    char * to_soundslike(char * res, const char * str, int len = -1) const { 
-      return soundslike_->to_soundslike(res,str,len);
-    }
-
-    char * to_soundslike(char * res, const char * str, int len, WordInfo wi) const {
-      if (!have_soundslike_ && (wi & ALL_CLEAN)) return 0;
-      else return soundslike_->to_soundslike(res,str,len);
-    }
-
     const char * soundslike_chars() const {return soundslike_chars_.c_str();}
-
-    //
-    // Affix compression methods
-    //
+    const char * clean_chars() const {return clean_chars_.c_str();}
 
     const AffixMgr * affix() const {return affix_;}
 
     bool have_affix() const {return affix_;}
 
-    void munch(ParmStr word, GuessInfo * cl, bool cross = true) const {
-      if (affix_)
-        affix_->munch(word, cl, cross);
-    }
-      
-    WordAff * expand(ParmStr word, ParmStr aff, 
-                     ObjStack & buf, int limit = INT_MAX) const {
-      if (affix_)
-        return affix_->expand(word, aff, buf, limit);
-      else
-        return fake_expand(word, aff, buf);
-    }
-    WordAff * fake_expand(ParmStr word, ParmStr aff, ObjStack & buf) const;
-
-    //
-    // Repl
-    //
-
-    bool have_repl() const {return have_repl_;}
-
     SuggestReplEnumeration * repl() const {
       return new SuggestReplEnumeration(repls_.pbegin(), repls_.pend());}
+
+    char * to_soundslike(char * res, const char * str, int len = -1) const 
+    { return soundslike_->to_soundslike(res,str,len);}
+
+    void munch(ParmString word, CheckList * cl) const {affix_->munch(word, cl);}
+
+    WordAff * expand(ParmString word, ParmString aff, 
+                     ObjStack & buf, int limit = INT_MAX) const {
+      return affix_->expand(word, aff, buf, limit);
+    }
     
-    //
-    //
-    //
+    char * to_lower(char * res, const char * str) const {
+      while (*str) *res++ = to_lower(*str++); *res = '\0'; return res;}
+    char * to_upper(char * res, const char * str) const {
+      while (*str) *res++ = to_upper(*str++); *res = '\0'; return res;}
+    char * to_stripped(char * res, const char * str) const {
+      for (; *str; ++str) {
+        if (special(*str).any) ++str;
+        *res++ = to_stripped(*str);
+      }
+      *res = '\0';
+      return res;
+    }
+    char * to_clean(char * res, const char * str) const {
+      for (; *str; ++str) {
+        if (special(*str).any) ++str;
+        *res++ = to_clean(*str);
+      }
+      *res = '\0';
+      return res;
+    }
 
-    WordInfo get_word_info(ParmStr str) const;
-    
-    //
-    // fix_case
-    //
+    const char * to_lower(String & res, const char * str) const {
+      res.clear(); while (*str) res += to_lower(*str++); return res.str();}
+    const char * to_upper(String & res, const char * str) const {
+      res.clear(); while (*str) res += to_upper(*str++); return res.str();}
+    const char * to_stripped(String & res, const char * str) const {
+      res.clear();
+      for (; *str; ++str) {
+        if (special(*str).any) ++str;
+        res += to_stripped(*str);
+      }
+      return res.str();
+    }
+    const char * to_clean(String & res, const char * str) const {
+      res.clear();
+      for (; *str; ++str) {
+        if (special(*str).any) ++str;
+        res += to_clean(*str);
+      }
+      return res.str();
+    }
 
-    CasePattern case_pattern(ParmStr str) const;
+    bool is_lower(const char * str) const {
+      while (*str) {if (!is_lower(*str++)) return false;} return true;}
+    bool is_upper(const char * str) const {
+      while (*str) {if (!is_upper(*str++)) return false;} return true;}
+    bool is_stripped(const char * str) const {
+      while (*str) {if (!is_stripped(*str++)) return false;} return true;}
+    bool is_clean(const char * str) const {
+      while (*str) {if (!is_clean(*str++)) return false;} return true;}
 
-    CasePattern case_pattern(const char * str, unsigned size) const;
+    WordInfo get_word_info(ParmString str) const;
+    CasePattern case_pattern(ParmString str) const;
 
     void fix_case(CasePattern case_pattern, char * str)
     {
@@ -371,17 +277,7 @@ namespace aspeller {
     void fix_case(CasePattern case_pattern, 
                   char * res, const char * str) const;
     const char * fix_case(CasePattern case_pattern, 
-                          const char * str, String & buf) const;
-
-    //
-    //
-    //
-
-    CompoundWord split_word(const char * str, unsigned size, bool camel_case) const;
-
-    //
-    // for cache
-    //
+                          const char * str, String buf) const;
 
     static inline PosibErr<Language *> get_new(const String & lang, const Config * config) {
       StackPtr<Language> l(new Language());
@@ -407,14 +303,19 @@ namespace aspeller {
     operator bool () const {return lang;}
     int operator() (const char * a, const char * b) const
     { 
-      char x, y;
-      for (;;)
+      if (lang->special(*a).begin) ++a;
+      if (lang->special(*b).begin) ++b;
+      while (*a != '\0' && *b != '\0' 
+	     && lang->to_clean(*a) == lang->to_clean(*b)) 
       {
-        while (x = lang->to_clean(*a++), !x);
-        while (y = lang->to_clean(*b++), !y);
-        if (x == 0x10 || y == 0x10 || x != y) break;
+	++a, ++b;
+	if (lang->special(*a).middle) ++a;
+	if (lang->special(*b).middle) ++b;
       }
-      return static_cast<unsigned char>(x) - static_cast<unsigned char>(y);
+      if (lang->special(*a).end) ++a;
+      if (lang->special(*b).end) ++b;
+      return static_cast<unsigned char>(lang->to_clean(*a)) 
+	- static_cast<unsigned char>(lang->to_clean(*b));
     }
   };
 
@@ -427,7 +328,6 @@ namespace aspeller {
     }
   };
   
-  template <typename HASH_INT = size_t>
   struct InsensitiveHash {
     // hashes a string without regards to casing or special begin
     // or end characters
@@ -435,13 +335,15 @@ namespace aspeller {
     InsensitiveHash() {}
     InsensitiveHash(const Language * l)
 	: lang(l) {}
-    HASH_INT operator() (const char * s) const
+    size_t operator() (const char * s) const
     {
-      HASH_INT h = 0;
+      size_t h = 0;
       for (;;) {
+	if (lang->special(*s).any) ++s;
 	if (*s == 0) break;
-        unsigned char c = lang->to_clean(*s++);
-	if (c) h=5*h + c;
+	if (lang->char_type(*s) == Language::Letter)
+	  h=5*h + lang->to_clean(*s);
+	++s;
       }
       return h;
     }
@@ -450,78 +352,67 @@ namespace aspeller {
   struct SensitiveCompare {
     const Language * lang;
     bool case_insensitive;
-    bool ignore_accents; // unused
-    bool begin; // if not begin we are checking the end of the word
-    bool end;   // if not end we are checking the beginning of the word
-                // if both false we are checking the middle of a word
+    bool ignore_accents;
+    bool strip_accents;
     SensitiveCompare(const Language * l = 0) 
-      : lang(l), case_insensitive(false), ignore_accents(false),
-        begin(true), end(true) {}
+      : lang(l), case_insensitive(false)
+      , ignore_accents(false), strip_accents(false) {}
     bool operator() (const char * word, const char * inlist) const;
+    // Pre:
+    //   word == to_find as given by Language::InsensitiveEqual 
+    //   both word and inlist contain at least one letter as given by
+    //     lang->char_type
+    // Rules:
+    //   if begin inlist is a begin char then it must match begin word
+    //   if end   inlist is a end   char then it must match end word
+    //   chop all begin/end chars from the begin/end of word and inlist
+    //  unless ignore_accents
+    //   accents must match
+    //  unless case_insensitive
+    //   (note: there are 3 posssible casings lower, upper and title)
+    //   if is lower begin inlist then begin word can be any casing
+    //   if not                   then begin word must be the same case
+    //   if word is all upper than casing of inlist can be anything
+    //   otherwise the casing of tail begin and tail inlist must match
   };
 
-  struct CleanAffix {
+  struct ConvertWord {
     const Language * lang;
-    OStream * log;
-    MsgConv msgconv1;
-    MsgConv msgconv2;
-    CleanAffix(const Language * lang0, OStream * log0);
-    char * operator() (ParmStr word, char * aff);
-  };
-
-  class WordListIterator
-  {
-  public:
-    struct Value {
-      SimpleString word;
-      SimpleString aff;
-    };
-    WordListIterator(StringEnumeration * in,
-                     const Language * lang,
-                     OStream * log);
-    // init may set "norm-strict" to true which is why it is not const
-    PosibErr<void> init (Config & config);
-    // init_plain initialized the iterator to read in a plain word
-    // list without any affix flags, for simplicity it will expect the
-    // input to be utf-8.  It will also assume clean the words unless
-    // the `clean-words` option is explicitly specified.  Like init it
-    // may set "norm-strict" to true which is why it is not const
-    PosibErr<void> init_plain (Config & config);
-    const Value & operator*() const {return val;}
-    const Value * operator-> () const {return &val;}
-    PosibErr<bool> adv();
-  private:
-    bool have_affix;
-    bool validate_words;
-    bool validate_affixes;
-    bool clean_words;
-    bool skip_invalid_words;
-    bool clean_affixes;
-    StringEnumeration * in;
-    const Language * lang;
-    ConvEC iconv;
-    OStream * log;
-    Value val;
-    String data;
-    const char * orig;
-    char * str;
-    char * str_end;
-    CleanAffix clean_affix;
+    bool strip_accents;
+    ConvertWord(const Language * l = 0)
+      : lang(l), strip_accents(false) {}
+    void convert(ParmString in, String & out) const
+    {
+      if (!strip_accents) {
+	out += in;
+      } else {
+	for (unsigned int i = 0; i != in.size(); ++i)
+	  out += lang->to_plain(in[i]);
+      }
+    }
+    void convert(ParmString in, char * out) const
+    {
+      if (!strip_accents) {
+        memcpy(out, in, in.size() + 1);
+      } else {
+        unsigned int i = 0;
+	for (; i != in.size(); ++i)
+	  out[i] = lang->to_plain(in[i]);
+        out[i] = '\0';
+      }
+    }
   };
 
   String get_stripped_chars(const Language & l);
 
   String get_clean_chars(const Language & l);
   
-  PosibErr<void> check_if_sane(const Language & l, ParmStr word);
-  PosibErr<void> check_if_valid(const Language & l, ParmStr word);
-  PosibErr<void> validate_affix(const Language & l, ParmStr word, ParmStr aff);
+  PosibErr<void> check_if_valid(const Language & l, ParmString word);
 
-  bool find_language(Config & c);
-
-  PosibErr<Language *> new_language(const Config &, ParmStr lang = 0);
+  PosibErr<Language *> new_language(const Config &, ParmString lang = 0);
 
   PosibErr<void> open_affix_file(const Config &, FStream & o);
+  
 }
 
 
