@@ -2,6 +2,7 @@
 #include "config.hpp"
 #include "data.hpp"
 #include "file_util.hpp"
+#include "file_util.hpp"
 #include "fstream.hpp"
 #include "getdata.hpp"
 #include "string.hpp"
@@ -14,34 +15,55 @@ namespace {
   using namespace acommon;
   using namespace aspeller;
 
-  typedef Vector<Dict *> Wss;
+  typedef Vector<LocalDict> Wss;
 
   class MultiDictImpl : public Dictionary
   {
   public:
     MultiDictImpl() : Dictionary(multi_dict, "MultiDictImpl") {}
-    PosibErr<void> load(ParmString, Config &, DictList *, SpellerImpl *);
+    PosibErr<void> load(ParmString, const Config &, LocalDictList *, 
+                        SpellerImpl *, const LocalDictInfo *);
     DictsEnumeration * dictionaries() const;
   private:
     Wss wss;
   };
 
   PosibErr<void> MultiDictImpl::load(ParmString fn, 
-                                     Config & config, 
-                                     DictList * new_dicts,
-                                     SpellerImpl * speller)
+                                     const Config & config, 
+                                     LocalDictList * new_dicts,
+                                     SpellerImpl * speller,
+                                     const LocalDictInfo * li)
   {
     String dir = figure_out_dir("",fn);
     FStream in;
     RET_ON_ERR(in.open(fn, "r"));
     set_file_name(fn);
+    bool strip_accents;
+    if (config.have("strip-accents"))
+      strip_accents = config.retrieve_bool("strip-accents");
+    else if (li == 0)
+      strip_accents = false;
+    else
+      strip_accents = li->convert.strip_accents;
     String buf; DataPair d;
     while(getdata_pair(in, d, buf)) 
     {
-      if (d.key == "add") {
+      if (d.key == "strip-accents") {
+	if (config.have("strip-accents")) {
+	  // do nothing
+	} if (d.value == "true") {
+	  strip_accents = true;
+	} else if (d.value == "false") {
+	  strip_accents = false;
+	} else {
+	  return make_err(bad_value, "strip-accents", d.value, "true or false").with_file(fn, d.line_num);
+	}
+      } else if (d.key == "add") {
 
-        RET_ON_ERR_SET(add_data_set(d.value, config, new_dicts, speller, dir), Dict *, res);
-        RET_ON_ERR(set_check_lang(res->lang()->name(), config));
+	LocalDict res;
+	res.set(0, config, strip_accents);
+        RET_ON_ERR(add_data_set(d.value, config, res, new_dicts, speller, &res, dir));
+        RET_ON_ERR(set_check_lang(res.dict->lang()->name(), &config));
 	wss.push_back(res);
 
       } else {
@@ -56,13 +78,13 @@ namespace {
 
   struct Parms
   {
-    typedef Dict * Value;
+    typedef const LocalDict * Value;
     typedef Wss::const_iterator Iterator;
     Iterator end;
     Parms(Iterator e) : end(e) {}
     bool endf(Iterator i)   const {return i == end;}
     Value end_state()       const {return 0;}
-    Value deref(Iterator i) const {return *i;}
+    Value deref(Iterator i) const {return &*i;}
   };
 
   DictsEnumeration * MultiDictImpl::dictionaries() const
