@@ -10,52 +10,38 @@
 
 namespace aspeller {
   
-  class SimpileSoundslike : public Soundslike {
+  class GenericSoundslike : public Soundslike {
   private:
     const Language * lang;
-    char first[256];
-    char rest[256];
   public:
-    SimpileSoundslike(const Language * l) : lang(l) {}
+    GenericSoundslike(const Language * l) : lang(l) {}
 
-    PosibErr<void> setup(Conv &) {
-      memcpy(first, lang->sl_first_, 256);
-      memcpy(rest,  lang->sl_rest_, 256);
-      return no_err;
-    }
+    PosibErr<void> setup(Conv &) {return no_err;}
     
     String soundslike_chars() const {
       bool chars_set[256] = {0};
-      for (int i = 0; i != 256; ++i) 
-      {
-        char c = first[i];
-        if (c) chars_set[static_cast<unsigned char>(c)] = true;
-        c = rest[i];
-        if (c) chars_set[static_cast<unsigned char>(c)] = true;
-      }
       String     chars_list;
       for (int i = 0; i != 256; ++i) 
-      {
-        if (chars_set[i]) 
-          chars_list += static_cast<char>(i);
-      }
+	{
+	  char c = lang->to_sl(static_cast<char>(i));
+	  if (c) chars_set[static_cast<unsigned char>(c)] = true;
+	}
+      for (int i = 0; i != 256; ++i) 
+	{
+	  if (chars_set[i]) 
+	    chars_list += static_cast<char>(i);
+	}
       return chars_list;
     }
-    
+
     char * to_soundslike(char * res, const char * str, int size) const 
     {
-      char prev, cur = '\0';
-
-      const char * i = str;
-      while (*i) {
-        cur = first[static_cast<unsigned char>(*i++)];
-        if (cur) {*res++ = cur; break;}
-      }
-      prev = cur;
+      char prev = '\0';
+      char cur;
       
-      while (*i) {
-	cur = rest[static_cast<unsigned char>(*i++)];
-	if (cur && cur != prev) *res++ = cur;
+      for (const char * i = str; *i; ++i) {
+	cur = lang->to_sl(*i);
+	if (cur != '\0' && cur != prev) *res++ = cur;
 	prev = cur;
       }
       *res = '\0';
@@ -63,10 +49,10 @@ namespace aspeller {
     }
 
     const char * name () const {
-      return "simple";
+      return "generic";
     }
     const char * version() const {
-      return "2.0";
+      return "1.0";
     }
   };
 
@@ -95,31 +81,6 @@ namespace aspeller {
     }
   };
 
-  class StrippedSoundslike : public Soundslike {
-  private:
-    const Language * lang;
-  public:
-    StrippedSoundslike(const Language * l) : lang(l) {}
-
-    PosibErr<void> setup(Conv &) {return no_err;}
-    
-    String soundslike_chars() const {
-      return get_stripped_chars(*lang);
-    }
-
-    char * to_soundslike(char * res, const char * str, int size) const 
-    {
-      return lang->LangImpl::to_stripped(res, str);
-    }
-
-    const char * name() const {
-      return "stripped";
-    }
-    const char * version() const {
-      return "1.0";
-    }
-  };
-
   class PhonetSoundslike : public Soundslike {
 
     const Language * lang;
@@ -185,16 +146,12 @@ namespace aspeller {
                                         const Language * lang)
   {
     Soundslike * sl;
-    if (name == "simple" || name == "generic") {
-      sl = new SimpileSoundslike(lang);
-    } else if (name == "stripped") {
-      sl = new StrippedSoundslike(lang);
+    if (name == "generic") {
+      sl = new GenericSoundslike(lang);
     } else if (name == "none") {
       sl = new NoSoundslike(lang);
-    } else if (name == lang->name()) {
-      sl = new PhonetSoundslike(lang);
     } else {
-      abort(); // FIXME
+      sl = new PhonetSoundslike(lang);
     }
     PosibErrBase pe = sl->setup(iconv);
     if (pe.has_err()) {
@@ -204,5 +161,6 @@ namespace aspeller {
       return sl;
     }
   }
+
 }
 
