@@ -27,22 +27,21 @@ namespace acommon {
 namespace aspeller {
 
   class Language;
-  struct SensitiveCompare;
+  class SensitiveCompare;
   class Suggest;
 
   enum SpecialId {main_id, personal_id, session_id, 
                   personal_repl_id, none_id};
 
-  struct SpellerDict
+  struct SpellerDict : public LocalDict
   {
-    Dict *            dict;
     bool              use_to_check;
     bool              use_to_suggest;
     bool              save_on_saveall;
     SpecialId         special_id;
     SpellerDict     * next;
-    SpellerDict(Dict *);
-    SpellerDict(Dict *, const Config &, SpecialId id = none_id);
+    SpellerDict(LocalDict &);
+    SpellerDict(Dict *, const Language *, const Config &, SpecialId id = none_id);
     ~SpellerDict() {if (dict) dict->release();}
   };
 
@@ -72,7 +71,7 @@ namespace aspeller {
     //
     // Add a single dictionary that has not been previously added
     //
-    PosibErr<void> add_dict(SpellerDict *);
+    void add_dict(SpellerDict *);
 
     PosibErr<const WordList *> personal_word_list  () const;
     PosibErr<const WordList *> session_word_list   () const;
@@ -91,60 +90,36 @@ namespace aspeller {
     //
     // Spelling methods
     //
-
-    struct CompoundInfo {
-      short count;
-      short incorrect_count;
-      CheckInfo * first_incorrect;      
-      CompoundInfo() : count(0), incorrect_count(0), first_incorrect() {}
-    };
   
     PosibErr<bool> check(char * word, char * word_end, /* it WILL modify word */
-                         bool try_uppercase,
-			 unsigned run_together_limit,
-			 CheckInfo *, CheckInfo *, GuessInfo *, CompoundInfo * = NULL);
+			 unsigned int run_together_limit,
+			 CheckInfo *, GuessInfo *);
 
     PosibErr<bool> check(MutableString word) {
-      guess_info.reset();
-      return check(word.begin(), word.end(), false,
+      guess_info.reset(guesses);
+      return check(word.begin(), 
+		   word.end(), 
 		   unconditional_run_together_ ? run_together_limit_ : 0,
-		   check_inf, check_inf + 8, &guess_info);
+		   check_inf, &guess_info);
     }
     PosibErr<bool> check(ParmString word)
     {
-      size_t sz = word.size();
-      std::vector<char> w(sz+1);
-      memcpy(&*w.begin(), word.str(), sz+1);
-      return check(MutableString(&w.front(), sz));
-    }
-    PosibErr<bool> check(const char * word) {return check(ParmString(word));}
-    PosibErr<bool> check(const char * word, size_t sz)
-    {
-      std::vector<char> w(sz+1);
-      memcpy(&*w.begin(), word, sz);
-      w[sz] = '\0';
-      return check(MutableString(&w.front(), sz));
+      std::vector<char> w(word.size()+1);
+      strncpy(&*w.begin(), word, w.size());
+      return check(MutableString(&w.front(), w.size() - 1));
     }
 
-    CheckInfo * check_runtogether(char * word, char * word_end, /* it WILL modify word */
-                                  bool try_uppercase,
-                                  unsigned run_together_limit,
-                                  CheckInfo *, CheckInfo *,
-                                  GuessInfo *);
-    
-    bool check_single(char * word, /* it WILL modify word */
-                      bool try_uppercase,
-                      CheckInfo & ci, GuessInfo * gi);
+    PosibErr<bool> check(const char * word) {return check(ParmString(word));}
 
     bool check_affix(ParmString word, CheckInfo & ci, GuessInfo * gi);
 
     bool check_simple(ParmString, WordEntry &);
 
     const CheckInfo * check_info() {
-      if (check_inf[0].word.str)
+      if (check_inf[0].word)
         return check_inf;
-      else if (guess_info.head)
-        return guess_info.head;
+      else if (guess_info.num > 0)
+        return guesses + 1;
       else
         return 0;
     }
@@ -187,6 +162,11 @@ namespace aspeller {
     ClonePtr<Suggest>       intr_suggest_;
     unsigned int            ignore_count;
     bool                    ignore_repl;
+    bool                    unconditional_run_together_;
+    unsigned int            run_together_limit_;
+    const char *            run_together_middle_;
+    unsigned int            run_together_min_;
+    unsigned int            run_together_start_len_;
     String                  prev_mis_repl_;
     String                  prev_cor_repl_;
 
@@ -214,37 +194,20 @@ namespace aspeller {
 		     const char *, const char *) const;
 
     CheckInfo check_inf[8];
+    CheckInfo guesses[8];
     GuessInfo guess_info;
 
-    SensitiveCompare s_cmp;
-    SensitiveCompare s_cmp_begin;  // These (s_cmp_begin,middle,end)
-    SensitiveCompare s_cmp_middle; // are used by the affix code.
-    SensitiveCompare s_cmp_end;
+    struct WSInfo : public LocalDictInfo
+    {
+      const Dictionary * dict;
+    };
 
-    typedef Vector<const Dict *> WS;
+    typedef Vector<WSInfo> WS;
     WS check_ws, affix_ws, suggest_ws, suggest_affix_ws;
 
-    bool                    unconditional_run_together_;
-    unsigned int            run_together_limit_;
-    unsigned int            run_together_min_;
-
-    unsigned run_together_limit() const {
-      return unconditional_run_together_ ? run_together_limit_ : 0;
-    }
-
-    bool camel_case_;
-
     bool affix_info, affix_compress;
 
-    bool have_repl;
-
-    bool have_soundslike;
-
-    bool invisible_soundslike, soundslike_root_only;
-
-    bool fast_scan, fast_lookup;
-
-    bool run_together;
+    bool use_soundslike, fast_scan, fast_lookup;
 
   };
 
@@ -254,16 +217,10 @@ namespace aspeller {
     SpellerImpl::WS::const_iterator begin;
     SpellerImpl::WS::const_iterator end;
     inline LookupInfo(SpellerImpl * s, Mode m);
-    // returns 0 if nothing found
-    // 1 if a match is found
-    // -1 if a word is found but affix doesn't match and "gi"
-    int lookup (ParmString word, const SensitiveCompare * c, char aff, 
-                WordEntry & o, GuessInfo * gi) const;
+    bool lookup (ParmString word, WordEntry & o) const;
   };
 
-  inline LookupInfo::LookupInfo(SpellerImpl * s, Mode m) 
-    : sp(s), mode(m) 
-  {
+  inline LookupInfo::LookupInfo(SpellerImpl * s, Mode m) : sp(s), mode(m) {
     switch (m) { 
     case Word: 
       begin = sp->affix_ws.begin(); 
